﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Media;
using Telerik.Windows.Controls;
using Telerik.Windows.Diagrams.Core;

namespace Diagram_WPF_CustomPolylineTool
{
    public class PolylineShapeTool : ToolBase, IMouseListener, IKeyboardListener
    {
        public PolylineShapeTool(string name):base(name)
        {
            this.Name = name;
        }

        IConnection currentConnection = null;
        Point lastPoint;

        public bool KeyDown(KeyArgs key)
        {
            if (IsActive && key.Key == System.Windows.Input.Key.Enter)
            {
                this.currentConnection.EndPoint = this.lastPoint;
                this.currentConnection.Update();
                this.DeactivateTool();
                return true;
            }
            return false;
        }

        public bool KeyUp(KeyArgs key)
        {
            return false;
        }

        public virtual bool MouseDown(PointerArgs e)
        {
            if (this.ToolService.ActiveTool != null && this.ToolService.ActiveTool.Name.Equals(this.Name))
            {
                if (this.currentConnection == null)
                {
                    this.currentConnection = new RadDiagramConnection();
                    this.currentConnection.StartPoint = e.Point;
                    this.currentConnection.EndPoint = this.lastPoint = e.Point;
                    this.Graph.Items.Add(this.currentConnection);
                    return true;
                }
                else
                {
                    this.currentConnection.ConnectionPoints.Add(lastPoint);
                    this.currentConnection.EndPoint = this.lastPoint = e.Point;
                    this.currentConnection.Update();
                    return true;
                }
            }
            return false;
        }

        protected override void OnActivated()
        {
            base.OnActivated();

            this.lastPoint = new Point(0, 0);
            this.currentConnection = null;
        }

        protected override void OnDeactivated()
        {
            base.OnDeactivated();

            this.ToolService.ActivateTool("Pointer Tool");

            this.Graph.Items.Remove(this.currentConnection);
            this.Graph.Items.Add(new RadDiagramShape()
            {
                Geometry = this.currentConnection.Geometry,
                Position = this.currentConnection.Position
            });
        }

        public virtual bool MouseMove(PointerArgs e)
        {
            if (this.currentConnection != null && this.IsActive)
            {
                this.currentConnection.EndPoint = this.lastPoint = e.Point;
                this.currentConnection.Update();
                return true;
            }
            return false;
        }

        public virtual bool MouseUp(PointerArgs e)
        {
            return false;
        }

        public bool MouseDoubleClick(PointerArgs e)
        {
            throw new NotImplementedException();
        }
    }
}
